Package: doppelgangR
Title: Identify likely duplicate samples from genomic or meta-data
Version: 1.41.0
Authors@R: c(person("Levi", "Waldron", email = "lwaldron.research@gmail.com",
    role = c("aut", "cre")),
    person("Markus", "Reister", email = "markus.riester@novartis.com",
    role = c("aut", "ctb")),
    person("Marcel", "Ramos", email = "marcel.ramos@sph.cuny.edu",
    role = "ctb"))
Description: The main function is doppelgangR(), which takes as minimal
    input a list of ExpressionSet object, and searches all list
    pairs for duplicated samples.  The search is based on the
    genomic data (exprs(eset)), phenotype/clinical data
    (pData(eset)), and "smoking guns" - supposedly unique
    identifiers found in pData(eset).
Depends: R (>= 3.5.0), Biobase, BiocParallel
Imports: sva, impute, digest, mnormt, methods, grDevices, graphics,
        stats, SummarizedExperiment, utils
Suggests: BiocStyle, knitr, rmarkdown, curatedOvarianData, testthat
biocViews: ImmunoOncology, RNASeq, Microarray, GeneExpression,
        QualityControl
License: GPL (>=2.0)
Encoding: UTF-8
URL: https://github.com/lwaldron/doppelgangR,
        https://waldronlab.github.io/doppelgangR
BugReports: https://github.com/lwaldron/doppelgangR/issues
VignetteBuilder: knitr
RoxygenNote: 7.1.0
Date: 2025-05-12
git_url: https://git.bioconductor.org/packages/doppelgangR
git_branch: devel
git_last_commit: 039348e
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.24
Date/Publication: 2026-04-28
NeedsCompilation: no
Packaged: 2026-04-28 22:24:41 UTC; biocbuild
Author: Levi Waldron [aut, cre],
  Markus Reister [aut, ctb],
  Marcel Ramos [ctb]
Maintainer: Levi Waldron <lwaldron.research@gmail.com>
